﻿Module mdlGame
    Public Sub NewGame(ByVal Player1Type As enmPlayerType, ByVal Player1Strength As enmCPUStrength, ByVal Player2Type As enmPlayerType, ByVal Player2Strength As enmCPUStrength)
        Dim RandomNum As New Random

        Call ClearMatrix()
        MoveCount = 0
        ActivePlayer = RandomNum.Next(0, 2)
        GameState = enmGameState.ActiveGame

        Players(ActivePlayer).Name = "X"
        Players(GetOpponentID(ActivePlayer)).Name = "O"

        Num2Sym(0) = " "
        If ActivePlayer = 0 Then
            Num2Sym(1) = "X"
            Num2Sym(4) = "O"
        Else
            Num2Sym(1) = "O"
            Num2Sym(4) = "X"
        End If

        Players(0).ID = 0
        Players(1).ID = 1

        Players(0).Type = Player1Type
        Players(1).Type = Player2Type

        Players(0).Strength = Player1Strength
        Players(1).Strength = Player2Strength

        Call DrawField()
        Call NextMove()
    End Sub

    Public Function MakeMove(ByVal Player As strPlayer, ByVal Location As Point) As enmMoveCheck
        If GameState = enmGameState.ActiveGame Then
            If FullMatrix(Location.X, Location.Y) = 0 And MoveCount < 9 Then
                PlayerMatrix(Player.ID, Location.X, Location.Y) = 1
                FullMatrix(Location.X, Location.Y) = 1 + Player.ID * 3 'Could be an If/Else construct but...
                ActivePlayer = GetOpponentID(ActivePlayer)
                Call DrawField()
                Application.DoEvents()
                If Players(0).Type = enmPlayerType.CPU And Players(1).Type = enmPlayerType.CPU Then
                    System.Threading.Thread.Sleep(1000)
                End If

                If CheckWin(Player.ID) = False Then
                    MakeMove = enmMoveCheck.Valid
                Else
                    MakeMove = enmMoveCheck.Win
                    GameState = enmGameState.NoGame
                    Redraw("P" & (Player.ID + 1) & ": " & Player.Name & " " & "wins")
                End If

                MoveCount += 1
                If MoveCount = 9 And MakeMove = enmMoveCheck.Valid Then
                    GameState = enmGameState.NoGame
                    Redraw("Draw")
                End If
            Else
                MakeMove = enmMoveCheck.Invalid
                Call DrawField()
                Redraw("P" & (Player.ID + 1) & ": " & Player.Name & " " & "invalid move")
            End If

            If Not MakeMove = enmMoveCheck.Win And MoveCount < 9 Then Call NextMove()
        End If
    End Function

    Private Sub NextMove()
        If Players(ActivePlayer).Type = enmPlayerType.CPU Then
            Call MakeMove(Players(ActivePlayer), CreateAiMove(Players(ActivePlayer)))
        Else
            'Call HumanInput(Players(ActivePlayer))
        End If
    End Sub

    Private Sub HumanInput(ByVal Player As strPlayer)
        Dim tempPoint As Point = AiEasy(Player.ID)
        tempPoint.X = CInt(InputBox("Give me the row (0,1,2)" & vbNewLine & frmChameleon.TextBox1.Text, "Row", tempPoint.X))
        tempPoint.Y = CInt(InputBox("Give me the col (0,1,2)" & vbNewLine & frmChameleon.TextBox1.Text, "Col", tempPoint.Y))
        MakeMove(Player, PositionFix(tempPoint))
    End Sub

    Private Function PositionFix(ByVal Position As Point) As Point
        If Position.X < 0 Then
            Position.X = 0
        ElseIf Position.X > 2 Then
            Position.X = 2
        End If
        If Position.Y < 0 Then
            Position.Y = 0
        ElseIf Position.Y > 2 Then
            Position.Y = 2
        End If
        Return Position
    End Function

    Public Function CheckWin(ByVal PlayerID As Byte) As Boolean
        Dim LineSum(,) As Byte = GetPlayerLineWeight(PlayerID)
        Dim LineWin(3, 2) As Boolean

        For I = 0 To 1
            For J = 0 To 2
                If LineSum(I, J) = 3 Then
                    LineWin(I, J) = True
                    CheckWin = True
                End If
            Next
        Next
        For I = 2 To 3
            If LineSum(I, 0) = 3 Then
                LineWin(I, 0) = True
                CheckWin = True
            End If
        Next

        GlobalLineSum = LineSum.Clone
    End Function

    Private Sub ClearMatrix()
        For I = 0 To 2
            For J = 0 To 2
                FullMatrix(I, J) = 0
                For K = 0 To 1
                    PlayerMatrix(K, I, J) = 0
                Next
            Next
        Next
    End Sub

    Public Function GetPlayerLineWeight(ByVal PlayerID As Byte) As Array
        Dim LineSum(3, 2) As Byte
        For I = 0 To 2
            For J = 0 To 2
                LineSum(0, I) += PlayerMatrix(PlayerID, I, J)
                LineSum(1, I) += PlayerMatrix(PlayerID, J, I)
            Next
            LineSum(2, 0) += PlayerMatrix(PlayerID, I, I)
            LineSum(3, 0) += PlayerMatrix(PlayerID, I, 2 - I)
        Next
        Return LineSum
    End Function

    Public Function GetOpponentMulti(ByVal PlayerID As Byte) As Byte
        If PlayerID = 1 Then
            Return 4
        Else
            Return 1
        End If
    End Function

    Public Function GetOpponentID(ByVal PlayerID As Byte) As Byte
        If PlayerID = 1 Then
            Return 0
        Else
            Return 1
        End If
    End Function

    Public Function GetLastField(ByVal LineType As Byte, ByVal LinePos As Byte) As Point
        Select Case LineType
            Case 1
                For I = 0 To 2
                    If FullMatrix(I, LinePos) = 0 Then
                        Return New Point(I, LinePos)
                    End If
                Next
            Case 0
                For I = 0 To 2
                    If FullMatrix(LinePos, I) = 0 Then
                        Return New Point(LinePos, I)
                    End If
                Next
            Case 2
                For I = 0 To 2
                    If FullMatrix(I, I) = 0 Then
                        Return New Point(I, I)
                    End If
                Next
            Case 3
                For I = 0 To 2
                    If FullMatrix(I, 2 - I) = 0 Then
                        Return New Point(I, 2 - I)
                    End If
                Next
        End Select
    End Function
End Module
