﻿
Module mdlGraphics
    Public BackBufferBitMap As Bitmap
    Public BackBuffer As System.Drawing.Graphics

    Public LoadPictureBackBufferBitMap As Bitmap
    Public LoadPictureBackBuffer As System.Drawing.Graphics

    Public SquareBackBufferBitMap As Bitmap
    Public SquareBackBuffer As System.Drawing.Graphics

    Public Sub CreateSquareBackBuffer()
        'If it is "Nothing" already no reset required
        If Not SquareBackBufferBitMap Is Nothing Then
            'Clear RAM and map it to nothing
            SquareBackBufferBitMap.Dispose()
            SquareBackBuffer.Dispose()
            SquareBackBufferBitMap = Nothing
            SquareBackBuffer = Nothing
        End If
        'Create a Backbuffer against ugly flickering
        SquareBackBufferBitMap = New Bitmap(SquareSize.Width, SquareSize.Height)
        SquareBackBuffer = Graphics.FromImage(SquareBackBufferBitMap)
    End Sub

    Public Sub CreateLoadPictureBackBuffer()
        'If it is "Nothing" already no reset required
        If Not LoadPictureBackBufferBitMap Is Nothing Then
            'Clear RAM and map it to nothing
            LoadPictureBackBufferBitMap.Dispose()
            LoadPictureBackBuffer.Dispose()
            LoadPictureBackBufferBitMap = Nothing
            LoadPictureBackBuffer = Nothing
        End If
        'Create a Backbuffer against ugly flickering
        LoadPictureBackBufferBitMap = New Bitmap(SquareSize.Width / 3, SquareSize.Width / 3)
        LoadPictureBackBuffer = Graphics.FromImage(LoadPictureBackBufferBitMap)
    End Sub

    Public Sub CreateBackBuffer()
        'If it is "Nothing" already no reset required
        If Not BackBufferBitMap Is Nothing Then
            'Clear RAM and map it to nothing
            BackBufferBitMap.Dispose()
            BackBuffer.Dispose()
            BackBufferBitMap = Nothing
            BackBuffer = Nothing
        End If
        'Create a Backbuffer against ugly flickering
        BackBufferBitMap = New Bitmap(DrawSpace.Width, DrawSpace.Height)
        BackBuffer = Graphics.FromImage(BackBufferBitMap)
        Call CreateSquareBackBuffer()
        Call CreateLoadPictureBackBuffer()
    End Sub

    Public Sub DrawField()
        Call SquareBackBuffer.Clear(Color.Black)
        For I = 1 To 2
            SquareBackBuffer.DrawLine(New Pen(Color.Gray), SquareSize.Width / 3 * I, 0, SquareSize.Width / 3 * I, SquareSize.Height)
            SquareBackBuffer.DrawLine(New Pen(Color.Gray), 0, SquareSize.Height / 3 * I, SquareSize.Width, SquareSize.Height / 3 * I)
        Next
        For I = 0 To 2
            For J = 0 To 2
                If Num2Sym(FullMatrix(J, I)) = "X" Then
                    If BackgroundPicturesExists(1) = True Then
                        SquareBackBuffer.DrawImage(BackgroundPicturesParts(1, J, I), J * SquareSize.Width / 3, I * SquareSize.Height / 3)
                    Else
                        SquareBackBuffer.DrawLine(New Pen(Color.LightGreen), SquareSize.Width / 3 * J + SquareSize.Width / 20, SquareSize.Height / 3 * I + SquareSize.Width / 20, SquareSize.Width / 3 * (J + 1) - SquareSize.Width / 20, SquareSize.Height / 3 * (I + 1) - SquareSize.Width / 20)
                        SquareBackBuffer.DrawLine(New Pen(Color.LightGreen), SquareSize.Width / 3 * (J + 1) - SquareSize.Width / 20, SquareSize.Height / 3 * I + SquareSize.Width / 20, SquareSize.Width / 3 * J + SquareSize.Width / 20, SquareSize.Height / 3 * (I + 1) - SquareSize.Width / 20)
                    End If
                ElseIf Num2Sym(FullMatrix(J, I)) = "O" Then
                    If BackgroundPicturesExists(2) = True Then
                        SquareBackBuffer.DrawImage(BackgroundPicturesParts(2, J, I), J * SquareSize.Width / 3, I * SquareSize.Height / 3)
                    Else
                        SquareBackBuffer.DrawEllipse(New Pen(Color.LightBlue), SquareSize.Width / 3 * J + SquareSize.Width / 20, SquareSize.Height / 3 * I + SquareSize.Width / 20, SquareSize.Width / 3 - SquareSize.Width / 10, SquareSize.Height / 3 - SquareSize.Width / 10)
                    End If
                Else
                    If BackgroundPicturesExists(0) = True Then
                        SquareBackBuffer.DrawImage(BackgroundPicturesParts(0, J, I), J * SquareSize.Width / 3, I * SquareSize.Height / 3)
                    End If
                End If
            Next
        Next
        Call Redraw()
    End Sub

    Public Sub DrawFieldDebug()
        frmChameleon.TextBox1.Text = vbNewLine
        Debug.WriteLine("")
        For I = 0 To 2
            frmChameleon.TextBox1.Text &= " "
            Debug.Write(" ")
            For J = 0 To 2
                frmChameleon.TextBox1.Text &= "[" & Num2Sym(FullMatrix(J, I)) & Int((FullMatrix(J, I) - 1) / 3 + 1) & "]" & " "
                Debug.Write("[" & Num2Sym(FullMatrix(J, I)) & Int((FullMatrix(J, I) - 1) / 3 + 1) & "]" & " ")
            Next
            frmChameleon.TextBox1.Text &= vbNewLine
            Debug.WriteLine("")
        Next
    End Sub

    Public Sub Redraw(Optional ByVal AlternativeMessage As String = "")
        Dim DrawString As String
        Dim StringSize As SizeF
        BackBuffer.Clear(Color.Black)
        BackBuffer.DrawImage(SquareBackBufferBitMap, DrawSpace.Width / 2 - SquareSize.Width / 2, (DrawSpace.Height - 2 * Config.ScreenSizeMulti - 2 * LineHeight) / 2 - SquareSize.Height / 2)

        If AlternativeMessage = "" Then
            DrawString = "P" & (ActivePlayer + 1) & ": " & Players(ActivePlayer).Name & " turns"
        Else
            DrawString = AlternativeMessage
        End If

        StringSize = BackBuffer.MeasureString(DrawString, frmChameleon.Font)

        BackBuffer.DrawString(DrawString, frmChameleon.Font, New SolidBrush(Color.Gray), DrawSpace.Width / 2 - StringSize.Width / 2 + Config.ScreenSizeMulti, DrawSpace.Height - Config.ScreenSizeMulti - 2 * LineHeight + LineHeight / 2)
        BackBuffer.DrawString(DrawString, frmChameleon.Font, New SolidBrush(Color.White), DrawSpace.Width / 2 - StringSize.Width / 2, DrawSpace.Height - 2 * Config.ScreenSizeMulti - 2 * LineHeight + LineHeight / 2)

        frmChameleon.pctDraw.Image = BackBufferBitMap
        frmChameleon.pctDraw.Refresh()
    End Sub

    Public Sub LoadPictures()
        Dim BackgroundNames() As String = {"background", "cross", "circle"}
        Dim CurrentPath As String
        For I = 0 To 2
            CurrentPath = AppPath & "\gfx\" & Config.Skin & "\" & Config.Skin & SquareSize.Width & "-" & BackgroundNames(I) & ".jpg"
            If System.IO.File.Exists(CurrentPath) Then
                BackgroundPictures(I) = New Bitmap(CurrentPath)
                BackgroundPicturesExists(I) = True
            Else
                BackgroundPicturesExists(I) = False
            End If
        Next

        For I = 0 To 2
            If BackgroundPicturesExists(I) = True Then
                For J = 0 To 2
                    For K = 0 To 2
                        LoadPictureBackBuffer.DrawImage(BackgroundPictures(I), -J * SquareSize.Width / 3, -K * SquareSize.Height / 3)
                        BackgroundPicturesParts(I, J, K) = LoadPictureBackBufferBitMap.Clone
                    Next
                Next
            End If
        Next
    End Sub

End Module
